@@ -26,7 +26,7 @@
 
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <libmissioncontrol/mc-account.h>
 #include <telepathy-glib/util.h>
@@ -492,32 +492,19 @@ widget_avatar_button_press_event_cb (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+update_avatar_chooser_account_cb (EmpathyAccountChooser *account_chooser,
+                                  EmpathyAvatarChooser *avatar_chooser)
+{
+  McAccount *account;
+
+  account = empathy_account_chooser_get_account (account_chooser);
+  g_object_set (avatar_chooser, "account", account, NULL);
+}
+
 static void
 contact_widget_contact_setup (EmpathyContactWidget *information)
 {
-  if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_AVATAR)
-    {
-      information->widget_avatar = empathy_avatar_chooser_new ();
-      g_signal_connect (information->widget_avatar, "changed",
-            G_CALLBACK (contact_widget_avatar_changed_cb),
-            information);
-    }
-  else
-    {
-      information->widget_avatar = empathy_avatar_image_new ();
-
-      g_signal_connect (information->widget_avatar, "popup-menu",
-          G_CALLBACK (widget_avatar_popup_menu_cb), information);
-      g_signal_connect (information->widget_avatar, "button-press-event",
-          G_CALLBACK (widget_avatar_button_press_event_cb), information);
-    }
-  
-  gtk_box_pack_start (GTK_BOX (information->vbox_avatar),
-          information->widget_avatar,
-          FALSE, FALSE,
-          6);
-  gtk_widget_show (information->widget_avatar);
-
   /* Setup account label/chooser */
   if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT)
     {
@@ -530,7 +517,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
   else
     {
       information->widget_account = gtk_label_new (NULL);
-      if (!information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP) {
+      if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP)) {
         gtk_label_set_selectable (GTK_LABEL (information->widget_account), TRUE);
       }
       gtk_misc_set_alignment (GTK_MISC (information->widget_account), 0, 0.5);
@@ -540,6 +527,39 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
            1, 2, 0, 1);
   gtk_widget_show (information->widget_account);
 
+  /* Set up avatar chooser/display */
+  if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_AVATAR)
+    {
+      information->widget_avatar = empathy_avatar_chooser_new ();
+      g_signal_connect (information->widget_avatar, "changed",
+            G_CALLBACK (contact_widget_avatar_changed_cb),
+            information);
+      if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT)
+        {
+          g_signal_connect (information->widget_account, "changed",
+              G_CALLBACK (update_avatar_chooser_account_cb),
+              information->widget_avatar);
+          update_avatar_chooser_account_cb (
+              EMPATHY_ACCOUNT_CHOOSER (information->widget_account),
+              EMPATHY_AVATAR_CHOOSER (information->widget_avatar));
+        }
+    }
+  else
+    {
+      information->widget_avatar = empathy_avatar_image_new ();
+
+      g_signal_connect (information->widget_avatar, "popup-menu",
+          G_CALLBACK (widget_avatar_popup_menu_cb), information);
+      g_signal_connect (information->widget_avatar, "button-press-event",
+          G_CALLBACK (widget_avatar_button_press_event_cb), information);
+    }
+
+  gtk_box_pack_start (GTK_BOX (information->vbox_avatar),
+          information->widget_avatar,
+          FALSE, FALSE,
+          6);
+  gtk_widget_show (information->widget_avatar);
+
   /* Setup id label/entry */
   if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_ID)
     {
@@ -554,7 +574,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
   else
     {
       information->widget_id = gtk_label_new (NULL);
-      if (!information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP) {
+      if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP)) {
         gtk_label_set_selectable (GTK_LABEL (information->widget_id), TRUE);
       }
       gtk_misc_set_alignment (GTK_MISC (information->widget_id), 0, 0.5);
@@ -578,7 +598,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
   else
     {
       information->widget_alias = gtk_label_new (NULL);
-      if (!information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP) {
+      if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP)) {
         gtk_label_set_selectable (GTK_LABEL (information->widget_alias), TRUE);
       }
       gtk_misc_set_alignment (GTK_MISC (information->widget_alias), 0, 0.5);
