@@ -22,6 +22,7 @@
  *          Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
+ *          Jonny Lamb <jonny.lamb@collabora.co.uk>
  * 
  *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
  *          Paolo Maggi
@@ -33,7 +34,7 @@
 #include <string.h>
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glade/glade.h>
@@ -45,6 +46,23 @@
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
+#include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-dispatcher.h>
+
+void
+empathy_gtk_init (void)
+{
+	static gboolean initialized = FALSE;
+
+	if (initialized)
+		return;
+
+	empathy_init ();
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   PKGDATADIR G_DIR_SEPARATOR_S "icons");
+
+	initialized = TRUE;
+}
 
 struct SizeData {
 	gint     width;
@@ -247,9 +265,19 @@ empathy_icon_name_for_contact (EmpathyContact *contact)
 GdkPixbuf *
 empathy_pixbuf_from_data (gchar *data,
 			  gsize  data_size)
+{
+	return empathy_pixbuf_from_data_and_mime (data, data_size, NULL);
+}
+
+GdkPixbuf *
+empathy_pixbuf_from_data_and_mime (gchar  *data,
+				   gsize   data_size,
+				   gchar **mime_type)
 {
 	GdkPixbufLoader *loader;
+	GdkPixbufFormat *format;
 	GdkPixbuf       *pixbuf = NULL;
+	gchar          **mime_types;
 	GError          *error = NULL;
 
 	if (!data) {
@@ -260,23 +288,34 @@ empathy_pixbuf_from_data (gchar *data,
 	if (!gdk_pixbuf_loader_write (loader, data, data_size, &error)) {
 		DEBUG ("Failed to write to pixbuf loader: %s",
 			error ? error->message : "No error given");
-		g_clear_error (&error);
-		g_object_unref (loader);
-		return NULL;
+		goto out;
 	}
 	if (!gdk_pixbuf_loader_close (loader, &error)) {
 		DEBUG ("Failed to close pixbuf loader: %s",
 			error ? error->message : "No error given");
-		g_clear_error (&error);
-		g_object_unref (loader);
-		return NULL;
+		goto out;
 	}
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 	if (pixbuf) {
 		g_object_ref (pixbuf);
+
+		if (mime_type != NULL) {
+			format = gdk_pixbuf_loader_get_format (loader);
+			mime_types = gdk_pixbuf_format_get_mime_types (format);
+
+			*mime_type = g_strdup (*mime_types);
+			if (mime_types[1] != NULL) {
+				DEBUG ("Loader supports more than one mime "
+					"type! Picking the first one, %s",
+					*mime_type);
+			}
+			g_strfreev (mime_types);
+		}
 	}
 
+out:
+	g_clear_error (&error);
 	g_object_unref (loader);
 
 	return pixbuf;
@@ -486,7 +525,7 @@ empathy_pixbuf_scale_down_if_necessary (GdkPixbuf *pixbuf, gint max_size)
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
 
-	if (width > max_size || height > max_size) {
+	if (width > 0 && (width > max_size || height > max_size)) {
 		factor = (gdouble) max_size / MAX (width, height);
 
 		width = width * factor;
@@ -1308,7 +1347,8 @@ fixup_url (const gchar *url)
 }
 
 void
-empathy_url_show (const char *url)
+empathy_url_show (GtkWidget *parent,
+		  const char *url)
 {
 	gchar  *real_url;
 	GError *error = NULL;
@@ -1318,10 +1358,9 @@ empathy_url_show (const char *url)
 		url = real_url;
 	}
 
-	/* FIXME: this does not work for multihead, we should use
-	 * GdkAppLaunchContext or gtk_show_url, see bug #514396.
-	 */
-	g_app_info_launch_default_for_uri (url, NULL, &error);
+	gtk_show_uri (gtk_widget_get_screen (parent), url,
+		      gtk_get_current_event_time (), &error);
+
 	if (error) {
 		GtkWidget *dialog;
 
@@ -1347,7 +1386,7 @@ link_button_hook (GtkLinkButton *button,
 		  const gchar *link,
 		  gpointer user_data)
 {
-	empathy_url_show (link);
+	empathy_url_show (GTK_WIDGET (button), link);
 }
 
 GtkWidget *
@@ -1377,51 +1416,77 @@ empathy_toggle_button_set_state_quietly (GtkWidget *widget,
 	g_signal_handlers_unblock_by_func (widget, callback, user_data);
 }
 
-GtkTextTag *
-empathy_text_buffer_tag_set (GtkTextBuffer *buffer,
-			     const gchar   *tag_name,
-			     const gchar   *first_property_name,
-			     ...)
+/* Sending files with the file chooser */
+
+static void
+file_manager_send_file_response_cb (GtkDialog      *widget,
+				    gint            response_id,
+				    EmpathyContact *contact)
 {
-	GtkTextTagTable *table;
-	GtkTextTag      *tag;
+	if (response_id == GTK_RESPONSE_OK) {
+		GSList *list;
+		GSList *l;
 
-	g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), NULL);
-	g_return_val_if_fail (tag_name != NULL, NULL);
+		list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (widget));
 
-	table = gtk_text_buffer_get_tag_table (buffer);
-	tag = gtk_text_tag_table_lookup (table, tag_name);
+		DEBUG ("File chooser selected files:");
 
-	if (!tag) {
-		tag = gtk_text_tag_new (tag_name);
-		gtk_text_tag_table_add (table, tag);
-		g_object_unref (tag);
-	} else {
-		/* Clear the old values so that we don't affect the new theme. */
-		g_object_set (tag,
-			      "background-set", FALSE,
-			      "foreground-set", FALSE,
-			      "invisible-set", FALSE,
-			      "justification-set", FALSE,
-			      "paragraph-background-set", FALSE,
-			      "pixels-above-lines-set", FALSE,
-			      "pixels-below-lines-set", FALSE,
-			      "rise-set", FALSE,
-			      "scale-set", FALSE,
-			      "size-set", FALSE,
-			      "style-set", FALSE,
-			      "weight-set", FALSE,
-			      NULL);
+		for (l = list; l; l = l->next) {
+			gchar            *uri;
+			GFile            *gfile;
+			GtkRecentManager *manager;
+
+			uri = l->data;
+			gfile = g_file_new_for_uri (uri);
+
+			DEBUG ("\t%s", uri);
+			empathy_dispatcher_send_file (contact, gfile);
+
+			manager = gtk_recent_manager_get_default ();
+			gtk_recent_manager_add_item (manager, uri);
+
+			g_object_unref (gfile);
+			g_free (uri);
+		}
+		g_slist_free (list);
 	}
 
-	if (first_property_name) {
-		va_list list;
+	gtk_widget_destroy (GTK_WIDGET (widget));
+}
 
-		va_start (list, first_property_name);
-		g_object_set_valist (G_OBJECT (tag), first_property_name, list);
-		va_end (list);
-	}
+void
+empathy_send_file_with_file_chooser (EmpathyContact *contact)
+{
+	GtkWidget               *widget;
+	GtkWidget               *button;
 
-	return tag;
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+	DEBUG ("Creating selection file chooser");
+
+	widget = gtk_file_chooser_dialog_new (_("Select a file"),
+					      NULL,
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL,
+					      GTK_RESPONSE_CANCEL,
+					      NULL);
+
+	/* send button */
+	button = gtk_button_new_with_mnemonic (_("_Send"));
+	gtk_button_set_image (GTK_BUTTON (button),
+		gtk_image_new_from_icon_name (EMPATHY_IMAGE_DOCUMENT_SEND,
+					      GTK_ICON_SIZE_BUTTON));
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (widget), button,
+				      GTK_RESPONSE_OK);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_dialog_set_default_response (GTK_DIALOG (widget),
+					 GTK_RESPONSE_OK);
+
+	g_signal_connect (widget, "response",
+			  G_CALLBACK (file_manager_send_file_response_cb),
+			  contact);
+
+	gtk_widget_show (widget);
 }
 
