@@ -28,7 +28,7 @@
  */
 
 #include "config.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "totem-subtitle-encoding.h"
 #include <string.h>
 
