@@ -23,33 +23,24 @@
 
 #include <string.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <dbus/dbus-glib.h>
 
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/util.h>
 #include <libmissioncontrol/mc-enum-types.h>
 
 #include "empathy-idle.h"
 #include "empathy-utils.h" 
+
+#define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include "empathy-debug.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       EMPATHY_TYPE_IDLE, EmpathyIdlePriv))
-
-#define DEBUG_DOMAIN "Idle"
-
 /* Number of seconds before entering extended autoaway. */
 #define EXT_AWAY_TIME (30*60)
 
-typedef enum {
-	NM_STATE_UNKNOWN,
-	NM_STATE_ASLEEP,
-	NM_STATE_CONNECTING,
-	NM_STATE_CONNECTED,
-	NM_STATE_DISCONNECTED
-} NMState;
-
-struct _EmpathyIdlePriv {
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyIdle)
+typedef struct {
 	MissionControl *mc;
 	DBusGProxy     *gs_proxy;
 	DBusGProxy     *nm_proxy;
@@ -60,40 +51,21 @@ struct _EmpathyIdlePriv {
 	gboolean        auto_away;
 	gboolean        use_nm;
 
-	gboolean        away_reset_status;
 	McPresence      away_saved_state;
-	gboolean        nm_reset_status;
 	McPresence      nm_saved_state;
 
 	gboolean        is_idle;
 	gboolean        nm_connected;
 	guint           ext_away_timeout;
-};
+} EmpathyIdlePriv;
 
-static void     empathy_idle_class_init      (EmpathyIdleClass *klass);
-static void     empathy_idle_init            (EmpathyIdle      *idle);
-static void     idle_finalize                (GObject          *object);
-static void     idle_get_property            (GObject          *object,
-					      guint             param_id,
-					      GValue           *value,
-					      GParamSpec       *pspec);
-static void     idle_set_property            (GObject          *object,
-					      guint             param_id,
-					      const GValue     *value,
-					      GParamSpec       *pspec);
-static void     idle_presence_changed_cb     (MissionControl   *mc,
-					      McPresence        state,
-					      gchar            *status,
-					      EmpathyIdle      *idle);
-static void     idle_session_idle_changed_cb (DBusGProxy       *gs_proxy,
-					      gboolean          is_idle,
-					      EmpathyIdle      *idle);
-static void     idle_nm_state_change_cb      (DBusGProxy       *proxy,
-					      guint             state,
-					      EmpathyIdle      *idle);
-static void     idle_ext_away_start          (EmpathyIdle      *idle);
-static void     idle_ext_away_stop           (EmpathyIdle      *idle);
-static gboolean idle_ext_away_cb             (EmpathyIdle      *idle);
+typedef enum {
+	NM_STATE_UNKNOWN,
+	NM_STATE_ASLEEP,
+	NM_STATE_CONNECTING,
+	NM_STATE_CONNECTED,
+	NM_STATE_DISCONNECTED
+} NMState;
 
 enum {
 	PROP_0,
@@ -107,113 +79,179 @@ enum {
 G_DEFINE_TYPE (EmpathyIdle, empathy_idle, G_TYPE_OBJECT)
 
 static void
-empathy_idle_class_init (EmpathyIdleClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->finalize = idle_finalize;
-	object_class->get_property = idle_get_property;
-	object_class->set_property = idle_set_property;
-
-	g_object_class_install_property (object_class,
-					 PROP_STATE,
-					 g_param_spec_enum ("state",
-							    "state",
-							    "state",
-							    MC_TYPE_PRESENCE,
-							    MC_PRESENCE_AVAILABLE,
-							    G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-					 PROP_STATUS,
-					 g_param_spec_string ("status",
-							      "status",
-							      "status",
-							      NULL,
-							      G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-					 PROP_FLASH_STATE,
-					 g_param_spec_enum ("flash-state",
-							    "flash-state",
-							    "flash-state",
-							    MC_TYPE_PRESENCE,
-							    MC_PRESENCE_UNSET,
-							    G_PARAM_READWRITE));
-
-	 g_object_class_install_property (object_class,
-					  PROP_AUTO_AWAY,
-					  g_param_spec_boolean ("auto-away",
-								"Automatic set presence to away",
-								"Should it set presence to away if inactive",
-								FALSE,
-								G_PARAM_READWRITE));
-
-	 g_object_class_install_property (object_class,
-					  PROP_USE_NM,
-					  g_param_spec_boolean ("use-nm",
-								"Use Network Manager",
-								"Set presence according to Network Manager",
-								FALSE,
-								G_PARAM_READWRITE));
-
-	g_type_class_add_private (object_class, sizeof (EmpathyIdlePriv));
-}
-
-static void
-empathy_idle_init (EmpathyIdle *idle)
+idle_presence_changed_cb (MissionControl *mc,
+			  McPresence      state,
+			  gchar          *status,
+			  EmpathyIdle    *idle)
 {
 	EmpathyIdlePriv *priv;
-	DBusGConnection *system_bus;
-	GError          *error = NULL;
 
 	priv = GET_PRIV (idle);
 
-	priv->is_idle = FALSE;
-	priv->mc = empathy_mission_control_new ();
-	priv->state = mission_control_get_presence_actual (priv->mc, NULL);
-	priv->status = mission_control_get_presence_message_actual (priv->mc, NULL);
+	DEBUG ("Presence changed to '%s' (%d)", status, state);
 
-	dbus_g_proxy_connect_signal (DBUS_G_PROXY (priv->mc),
-				     "PresenceChanged",
-				     G_CALLBACK (idle_presence_changed_cb),
-				     idle, NULL);
-
-	priv->gs_proxy = dbus_g_proxy_new_for_name (tp_get_bus (),
-						    "org.gnome.ScreenSaver",
-						    "/org/gnome/ScreenSaver",
-						    "org.gnome.ScreenSaver");
-	if (priv->gs_proxy) {
-		dbus_g_proxy_add_signal (priv->gs_proxy, "SessionIdleChanged",
-					 G_TYPE_BOOLEAN,
-					 G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal (priv->gs_proxy, "SessionIdleChanged",
-					     G_CALLBACK (idle_session_idle_changed_cb),
-					     idle, NULL);
-	} else {
-		empathy_debug (DEBUG_DOMAIN, "Failed to get gs proxy");
+	g_free (priv->status);
+	priv->state = state;
+	priv->status = NULL;
+	if (!G_STR_EMPTY (status)) {
+		priv->status = g_strdup (status);
 	}
 
+	g_object_notify (G_OBJECT (idle), "state");
+	g_object_notify (G_OBJECT (idle), "status");
+}
 
-	system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (!system_bus) {
-		empathy_debug (DEBUG_DOMAIN, "Failed to get system bus: %s",
-			      error ? error->message : "No error given");
-	} else {
-		priv->nm_proxy = dbus_g_proxy_new_for_name (system_bus,
-							    "org.freedesktop.NetworkManager",
-							    "/org/freedesktop/NetworkManager",
-							    "org.freedesktop.NetworkManager");
+static gboolean
+idle_ext_away_cb (EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+
+	priv = GET_PRIV (idle);
+
+	DEBUG ("Going to extended autoaway");
+	empathy_idle_set_state (idle, MC_PRESENCE_EXTENDED_AWAY);
+	priv->ext_away_timeout = 0;
+
+	return FALSE;
+}
+
+static void
+idle_ext_away_stop (EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+
+	priv = GET_PRIV (idle);
+
+	if (priv->ext_away_timeout) {
+		g_source_remove (priv->ext_away_timeout);
+		priv->ext_away_timeout = 0;
 	}
-	if (priv->nm_proxy) {
-		dbus_g_proxy_add_signal (priv->nm_proxy, "StateChange",
-					 G_TYPE_UINT, G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal (priv->nm_proxy, "StateChange",
-					     G_CALLBACK (idle_nm_state_change_cb),
-					     idle, NULL);
-	} else {
-		empathy_debug (DEBUG_DOMAIN, "Failed to get nm proxy");
+}
+
+static void
+idle_ext_away_start (EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+
+	priv = GET_PRIV (idle);
+
+	if (priv->ext_away_timeout != 0) {
+		return;
+	}
+	priv->ext_away_timeout = g_timeout_add_seconds (EXT_AWAY_TIME,
+							(GSourceFunc) idle_ext_away_cb,
+							idle);
+}
+
+static void
+idle_session_idle_changed_cb (DBusGProxy  *gs_proxy,
+			      gboolean     is_idle,
+			      EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+
+	priv = GET_PRIV (idle);
+
+	DEBUG ("Session idle state changed, %s -> %s",
+		priv->is_idle ? "yes" : "no",
+		is_idle ? "yes" : "no");
+
+	if (!priv->auto_away ||
+	    (priv->nm_saved_state == MC_PRESENCE_UNSET &&
+	     (priv->state <= MC_PRESENCE_OFFLINE ||
+	      priv->state == MC_PRESENCE_HIDDEN))) {
+		/* We don't want to go auto away OR we explicitely asked to be
+		 * offline, nothing to do here */
+		priv->is_idle = is_idle;
+		return;
 	}
 
-	priv->nm_connected = TRUE;
+	if (is_idle && !priv->is_idle) {
+		McPresence new_state;
+		/* We are now idle */
+
+		idle_ext_away_start (idle);
+
+		if (priv->nm_saved_state != MC_PRESENCE_UNSET) {
+		    	/* We are disconnected, when coming back from away
+		    	 * we want to restore the presence before the
+		    	 * disconnection. */
+			priv->away_saved_state = priv->nm_saved_state;
+		} else {
+			priv->away_saved_state = priv->state;
+		}
+
+		new_state = MC_PRESENCE_AWAY;
+		if (priv->state == MC_PRESENCE_EXTENDED_AWAY) {
+			new_state = MC_PRESENCE_EXTENDED_AWAY;
+		}
+
+		DEBUG ("Going to autoaway. Saved state=%d, new state=%d",
+			priv->away_saved_state, new_state);
+		empathy_idle_set_state (idle, new_state);
+	} else if (!is_idle && priv->is_idle) {
+		const gchar *new_status;
+		/* We are no more idle, restore state */
+
+		idle_ext_away_stop (idle);
+
+		if (priv->away_saved_state == MC_PRESENCE_AWAY ||
+		    priv->away_saved_state == MC_PRESENCE_EXTENDED_AWAY) {
+			priv->away_saved_state = MC_PRESENCE_AVAILABLE;
+			new_status = NULL;
+		} else {
+			new_status = priv->status;
+		}
+
+		DEBUG ("Restoring state to %d, reset status to %s",
+			priv->away_saved_state, new_status);
+
+		empathy_idle_set_presence (idle,
+					   priv->away_saved_state,
+					   new_status);
+
+		priv->away_saved_state = MC_PRESENCE_UNSET;
+	}
+
+	priv->is_idle = is_idle;
+}
+
+static void
+idle_nm_state_change_cb (DBusGProxy  *proxy,
+			 guint        state,
+			 EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+	gboolean         old_nm_connected;
+	gboolean         new_nm_connected;
+
+	priv = GET_PRIV (idle);
+
+	if (!priv->use_nm) {
+		return;
+	}
+
+	old_nm_connected = priv->nm_connected;
+	new_nm_connected = !(state == NM_STATE_CONNECTING ||
+			     state == NM_STATE_DISCONNECTED);
+	priv->nm_connected = TRUE; /* To be sure _set_state will work */
+
+	DEBUG ("New network state %d", state);
+
+	if (old_nm_connected && !new_nm_connected) {
+		/* We are no more connected */
+		DEBUG ("Disconnected: Save state %d", priv->state);
+		priv->nm_saved_state = priv->state;
+		empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
+	}
+	else if (!old_nm_connected && new_nm_connected) {
+		/* We are now connected */
+		DEBUG ("Reconnected: Restore state %d", priv->nm_saved_state);
+		empathy_idle_set_state (idle, priv->nm_saved_state);
+		priv->nm_saved_state = MC_PRESENCE_UNSET;
+	}
+
+	priv->nm_connected = new_nm_connected;
 }
 
 static void
@@ -301,6 +339,120 @@ idle_set_property (GObject      *object,
 	};
 }
 
+static void
+empathy_idle_class_init (EmpathyIdleClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	object_class->finalize = idle_finalize;
+	object_class->get_property = idle_get_property;
+	object_class->set_property = idle_set_property;
+
+	g_object_class_install_property (object_class,
+					 PROP_STATE,
+					 g_param_spec_enum ("state",
+							    "state",
+							    "state",
+							    MC_TYPE_PRESENCE,
+							    MC_PRESENCE_AVAILABLE,
+							    G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_STATUS,
+					 g_param_spec_string ("status",
+							      "status",
+							      "status",
+							      NULL,
+							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_FLASH_STATE,
+					 g_param_spec_enum ("flash-state",
+							    "flash-state",
+							    "flash-state",
+							    MC_TYPE_PRESENCE,
+							    MC_PRESENCE_UNSET,
+							    G_PARAM_READWRITE));
+
+	 g_object_class_install_property (object_class,
+					  PROP_AUTO_AWAY,
+					  g_param_spec_boolean ("auto-away",
+								"Automatic set presence to away",
+								"Should it set presence to away if inactive",
+								FALSE,
+								G_PARAM_READWRITE));
+
+	 g_object_class_install_property (object_class,
+					  PROP_USE_NM,
+					  g_param_spec_boolean ("use-nm",
+								"Use Network Manager",
+								"Set presence according to Network Manager",
+								FALSE,
+								G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof (EmpathyIdlePriv));
+}
+
+static void
+empathy_idle_init (EmpathyIdle *idle)
+{
+	DBusGConnection *system_bus;
+	GError          *error = NULL;
+	EmpathyIdlePriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (idle,
+		EMPATHY_TYPE_IDLE, EmpathyIdlePriv);
+
+	idle->priv = priv;
+	priv->is_idle = FALSE;
+	priv->mc = empathy_mission_control_new ();
+	priv->state = mission_control_get_presence_actual (priv->mc, NULL);
+	priv->status = mission_control_get_presence_message_actual (priv->mc, NULL);
+	if (G_STR_EMPTY (priv->status)) {
+		g_free (priv->status);
+		priv->status = NULL;
+	}
+
+	dbus_g_proxy_connect_signal (DBUS_G_PROXY (priv->mc),
+				     "PresenceChanged",
+				     G_CALLBACK (idle_presence_changed_cb),
+				     idle, NULL);
+
+	priv->gs_proxy = dbus_g_proxy_new_for_name (tp_get_bus (),
+						    "org.gnome.ScreenSaver",
+						    "/org/gnome/ScreenSaver",
+						    "org.gnome.ScreenSaver");
+	if (priv->gs_proxy) {
+		dbus_g_proxy_add_signal (priv->gs_proxy, "SessionIdleChanged",
+					 G_TYPE_BOOLEAN,
+					 G_TYPE_INVALID);
+		dbus_g_proxy_connect_signal (priv->gs_proxy, "SessionIdleChanged",
+					     G_CALLBACK (idle_session_idle_changed_cb),
+					     idle, NULL);
+	} else {
+		DEBUG ("Failed to get gs proxy");
+	}
+
+
+	system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (!system_bus) {
+		DEBUG ("Failed to get system bus: %s",
+			error ? error->message : "No error given");
+	} else {
+		priv->nm_proxy = dbus_g_proxy_new_for_name (system_bus,
+							    "org.freedesktop.NetworkManager",
+							    "/org/freedesktop/NetworkManager",
+							    "org.freedesktop.NetworkManager");
+	}
+	if (priv->nm_proxy) {
+		dbus_g_proxy_add_signal (priv->nm_proxy, "StateChange",
+					 G_TYPE_UINT, G_TYPE_INVALID);
+		dbus_g_proxy_connect_signal (priv->nm_proxy, "StateChange",
+					     G_CALLBACK (idle_nm_state_change_cb),
+					     idle, NULL);
+	} else {
+		DEBUG ("Failed to get nm proxy");
+	}
+
+	priv->nm_connected = TRUE;
+}
+
 EmpathyIdle *
 empathy_idle_new (void)
 {
@@ -398,24 +550,31 @@ empathy_idle_set_presence (EmpathyIdle *idle,
 
 	priv = GET_PRIV (idle);
 
-	empathy_debug (DEBUG_DOMAIN, "Changing presence to %s (%d)",
-		       status, state);
-
-	if (!priv->nm_connected) {
-		empathy_debug (DEBUG_DOMAIN, "NM not connected");
-		return;
-	}
+	DEBUG ("Changing presence to %s (%d)", status, state);
 
 	/* Do not set translated default messages */
 	default_status = empathy_presence_get_default_message (state);
-	if (status && strcmp (status, default_status) == 0) {
+	if (!tp_strdiff (status, default_status)) {
 		status = NULL;
 	}
 
-	mission_control_set_presence (priv->mc,
-				      state,
-				      status,
-				      NULL, NULL);
+	if (!priv->nm_connected) {
+		DEBUG ("NM not connected");
+
+		priv->nm_saved_state = state;
+		if (tp_strdiff (priv->status, status)) {
+			g_free (priv->status);
+			priv->status = NULL;
+			if (!G_STR_EMPTY (status)) {
+				priv->status = g_strdup (status);
+			}
+			g_object_notify (G_OBJECT (idle), "status");
+		}
+
+		return;
+	}
+
+	mission_control_set_presence (priv->mc, state, status, NULL, NULL);
 }
 
 gboolean
@@ -468,197 +627,20 @@ empathy_idle_set_use_nm (EmpathyIdle *idle,
 				   G_TYPE_INVALID);
 
 		if (error) {
-			empathy_debug (DEBUG_DOMAIN, 
-				       "Couldn't get NM state: %s",
-				       error->message);
+			DEBUG ("Couldn't get NM state: %s", error->message);
 			g_clear_error (&error);
 			nm_status = NM_STATE_ASLEEP;
 		}
 		
 		idle_nm_state_change_cb (priv->nm_proxy, nm_status, idle);
 	} else {
-		if (!priv->nm_connected) {
+		priv->nm_connected = TRUE;
+		if (priv->nm_saved_state != MC_PRESENCE_UNSET) {
 			empathy_idle_set_state (idle, priv->nm_saved_state);
 		}
-		priv->nm_connected = TRUE;
 		priv->nm_saved_state = MC_PRESENCE_UNSET;
 	}
 
 	g_object_notify (G_OBJECT (idle), "use-nm");
 }
 
-static void
-idle_presence_changed_cb (MissionControl *mc,
-			  McPresence      state,
-			  gchar          *status,
-			  EmpathyIdle    *idle)
-{
-	EmpathyIdlePriv *priv;
-
-	priv = GET_PRIV (idle);
-
-	empathy_debug (DEBUG_DOMAIN, "Presence changed to '%s' (%d)",
-		       status, state);
-
-	g_free (priv->status);
-	priv->state = state;
-	priv->status = NULL;
-	if (!G_STR_EMPTY (status)) {
-		priv->status = g_strdup (status);
-	}
-
-	g_object_notify (G_OBJECT (idle), "state");
-	g_object_notify (G_OBJECT (idle), "status");
-}
-
-static void
-idle_session_idle_changed_cb (DBusGProxy  *gs_proxy,
-			      gboolean     is_idle,
-			      EmpathyIdle *idle)
-{
-	EmpathyIdlePriv *priv;
-
-	priv = GET_PRIV (idle);
-
-	empathy_debug (DEBUG_DOMAIN, "Session idle state changed, %s -> %s",
-		      priv->is_idle ? "yes" : "no",
-		      is_idle ? "yes" : "no");
-
-	if (priv->state <= MC_PRESENCE_OFFLINE ||
-	    priv->state == MC_PRESENCE_HIDDEN ||
-	    !priv->auto_away) {
-		/* We are not online or we don't want to go auto away,
-		 * nothing to do here */
-		priv->is_idle = is_idle;
-		return;
-	}
-
-	if (is_idle && !priv->is_idle) {
-		McPresence new_state;
-		/* We are now idle */
-
-		if (priv->state == MC_PRESENCE_AWAY ||
-		    priv->state == MC_PRESENCE_EXTENDED_AWAY) {
-			/* User set away manually, when coming back we restore
-			 * default presence. */
-			new_state = priv->state;
-			priv->away_saved_state = MC_PRESENCE_AVAILABLE;
-			priv->away_reset_status = TRUE;
-		} else {
-			new_state = MC_PRESENCE_AWAY;
-			priv->away_saved_state = priv->state;
-			priv->away_reset_status = FALSE;
-		}
-
-		empathy_debug (DEBUG_DOMAIN, "Going to autoaway");
-		empathy_idle_set_state (idle, new_state);
-
-		idle_ext_away_start (idle);
-	} else if (!is_idle && priv->is_idle) {
-		/* We are no more idle, restore state */
-		idle_ext_away_stop (idle);
-
-		empathy_debug (DEBUG_DOMAIN, "Restoring state to %d, reset status: %s",
-			      priv->away_saved_state,
-			      priv->away_reset_status ? "Yes" : "No");
-
-		if (priv->nm_connected) {
-			empathy_idle_set_presence (idle,
-						   priv->away_saved_state,
-						   priv->away_reset_status ? NULL : priv->status);
-		} else {
-			/* We can't restore state now, will do when NM gets
-			 * connected. */
-			priv->nm_saved_state = priv->away_saved_state;
-			priv->nm_reset_status = priv->away_reset_status;
-		}
-
-		priv->away_saved_state = MC_PRESENCE_UNSET;
-		priv->away_reset_status = FALSE;
-	}
-
-	priv->is_idle = is_idle;
-}
-
-static void
-idle_nm_state_change_cb (DBusGProxy  *proxy,
-			 guint        state,
-			 EmpathyIdle *idle)
-{
-	EmpathyIdlePriv *priv;
-	gboolean         old_nm_connected;
-	gboolean         new_nm_connected;
-
-	priv = GET_PRIV (idle);
-
-	empathy_debug (DEBUG_DOMAIN, "New network state (%d), in use = %s",
-		       state, priv->use_nm ? "Yes" : "No");
-
-	if (!priv->use_nm) {
-		return;
-	}
-
-	old_nm_connected = priv->nm_connected;
-	new_nm_connected = !(state == NM_STATE_CONNECTING ||
-			     state == NM_STATE_DISCONNECTED);
-	priv->nm_connected = TRUE; /* To be sure _set_state will work */
-
-	if (old_nm_connected && !new_nm_connected) {
-		/* We are no more connected */
-		idle_ext_away_stop (idle);
-
-		priv->nm_saved_state = priv->state;
-		empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
-	}
-	else if (!old_nm_connected && new_nm_connected) {
-		/* We are now connected */
-		empathy_idle_set_presence (idle,
-					   priv->nm_saved_state,
-					   priv->nm_reset_status ? NULL : priv->status);
-		priv->nm_saved_state = MC_PRESENCE_UNSET;
-		priv->nm_reset_status = FALSE;
-	}
-
-	priv->nm_connected = new_nm_connected;
-}
-
-static void
-idle_ext_away_start (EmpathyIdle *idle)
-{
-	EmpathyIdlePriv *priv;
-
-	priv = GET_PRIV (idle);
-
-	idle_ext_away_stop (idle);
-	priv->ext_away_timeout = g_timeout_add_seconds (EXT_AWAY_TIME,
-							(GSourceFunc) idle_ext_away_cb,
-							idle);
-}
-
-static void
-idle_ext_away_stop (EmpathyIdle *idle)
-{
-	EmpathyIdlePriv *priv;
-
-	priv = GET_PRIV (idle);
-
-	if (priv->ext_away_timeout) {
-		g_source_remove (priv->ext_away_timeout);
-		priv->ext_away_timeout = 0;
-	}
-}
-
-static gboolean
-idle_ext_away_cb (EmpathyIdle *idle)
-{
-	EmpathyIdlePriv *priv;
-
-	priv = GET_PRIV (idle);
-
-	empathy_debug (DEBUG_DOMAIN, "Going to extended autoaway");
-	empathy_idle_set_state (idle, MC_PRESENCE_EXTENDED_AWAY);
-	priv->ext_away_timeout = 0;
-
-	return FALSE;
-}
-
